import { __awaiter, __decorate } from "tslib";
import { customElement, LitElement, property, html, query } from 'lit-element';
import { unsafeHTML } from 'lit-html/directives/unsafe-html';
import 'prismjs';
import 'prismjs/components/prism-typescript';
import { tokenizeMarkup } from './directives/tokenize-markup';
import { resolveAlias } from './utils/template-aliases';
import { styles } from './style';
const languageConverter = { fromAttribute: resolveAlias };
const rangeConverter = { fromAttribute: (value) => value.split(',').map(s => s.split('-').map(Number)) };
const getFileContent = (src) => __awaiter(void 0, void 0, void 0, function* () {
    const response = yield fetch(src);
    if (!response || !response.ok) {
        throw new Error('bad response');
    }
    const content = yield response.text();
    if (!content) {
        throw new Error('bad content');
    }
    return content.trim();
});
let CodeHighlightElement = class CodeHighlightElement extends LitElement {
    constructor() {
        super(...arguments);
        this.plain = false;
    }
    get codeElement() {
        let el = this.querySelector('code');
        !el && this.appendChild((el = document.createElement('code')));
        return el;
    }
    get source() {
        return this.codeElement.innerText.trim();
    }
    connectedCallback() {
        super.connectedCallback();
        const observer = new MutationObserver(records => {
            const { codeElement } = this;
            const codeChange = !!records.find(record => record.target === codeElement);
            codeChange && this.requestUpdate();
        });
        observer.observe(this, { attributes: false, childList: true, subtree: true });
    }
    render() {
        var _a;
        const { language = 'clike', source } = this;
        return html `
    <pre>
      <span id="index">${this._renderIndex()}</span>
      <code>${tokenizeMarkup(source, language)}</code>
      ${(_a = this.highlight) === null || _a === void 0 ? void 0 : _a.map(([start, end]) => this._renderHighlight(start, end))}
    </pre>
    `;
    }
    _renderIndex() {
        const lines = this.source.split(/\n/gm);
        const index = unsafeHTML(new Array(lines.length + 1).join('<span></span>'));
        return index;
    }
    _renderHighlight(start, end = start) {
        var _a;
        const elements = (_a = this._index) === null || _a === void 0 ? void 0 : _a.children;
        console.log(start, end, elements);
        if (!elements || elements.length < end)
            return;
        const startElement = elements[start - 1];
        if (!(startElement instanceof HTMLElement))
            return;
        const endElement = elements[end - 1];
        if (!(endElement instanceof HTMLElement))
            return;
        const top = startElement.offsetTop;
        const bottom = endElement.offsetTop + endElement.clientHeight;
        return html `<span class="highlight" style="top: ${top}px; height: ${bottom - top}px"></span>`;
    }
    attributeChangedCallback(name, old, value) {
        super.attributeChangedCallback(name, old, value);
        if (name === 'src') {
            if (value) {
                getFileContent(value)
                    .then(code => {
                    this.language = value.split('.').pop();
                    this.codeElement.innerHTML = code;
                })
                    .catch(console.error);
            }
        }
    }
};
CodeHighlightElement.styles = styles;
__decorate([
    property()
], CodeHighlightElement.prototype, "src", void 0);
__decorate([
    property({ reflect: true, converter: languageConverter })
], CodeHighlightElement.prototype, "language", void 0);
__decorate([
    property({ reflect: true })
], CodeHighlightElement.prototype, "theme", void 0);
__decorate([
    property({ reflect: true })
], CodeHighlightElement.prototype, "cdn", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], CodeHighlightElement.prototype, "plain", void 0);
__decorate([
    property({ type: Array, converter: rangeConverter })
], CodeHighlightElement.prototype, "highlight", void 0);
__decorate([
    query("#index")
], CodeHighlightElement.prototype, "_index", void 0);
CodeHighlightElement = __decorate([
    customElement('cwc-code-highlight')
], CodeHighlightElement);
export { CodeHighlightElement };
export default CodeHighlightElement;
//# sourceMappingURL=code-highlight.js.map